/****************************************************************************
 * JCLCAT.C (c) copyright Jay Moseley, CCP 2004
 *          Processes Job Control Language (JCL) files with embedded
 *          directives to allow "nesting" of files; output to stdout
 *          for submission to Hercules socket reader (SUBMIT).
 *----------------------------------------------------------------------------
 * 2004/11/01 #1 Modified for loop that copies subfile name from the
 *               input file name; originally used blank as delimiter
 *               but found for unnumbered files SPF/PC did not replace
 *               null characters with blanks so the last character of
 *               the file name was followed by x'0d0a'.
 ****************************************************************************/

#include "ctype.h"
#include "stdio.h"
#include "stdlib.h"
#include "string.h"

const char includeKey[] = "$include";  /* directive for included files      */
const char endKey[] = "$end";          /* directive to early terminate file */

/****************************************************************************
 * processFile reads the passed file handle (opened before call) and either
 * handles the two directives that may be embedded in the file or prints
 * the record to stdout.
 ****************************************************************************/
void processFile(FILE *in, int level)         /* may be invoked recursively */
{
  char buffer[82];        /* buffer for reading the file                    */
  char byteSave;          /* to save byte displaced during comparisions     */
  char subFileName[100];  /* name of subfile extracted from $INCLUDE        */
  FILE *subFile;          /* handle for opening subfile                     */
  char lvlgraph[20];      /* used to graphically display file nesting level */
  int i, j;               /* loop counters for scanning record buffer       */

  /* this loop continues until end-of-file or $END directive read           */
  do
  {
    /* if end-of-file indicated, terminate loop                             */
    if (fgets(buffer, 82, in) == 0)
      break;

    /* if $END directive read, terminate loop                               */
    byteSave = buffer[4];
    buffer[4] = '\0';
    if (strcmpi(endKey, buffer) == 0)
      break;
    buffer[4] = byteSave;

    /* if $INCLUDE directive read, extract file name and process            */
    byteSave = buffer[8];
    buffer[8] = '\0';
    if (strcmpi(includeKey, buffer) == 0)
    {
      for (i=9; buffer[i] == ' '; i++) ; /* skip blanks                     */
      for (j=0; !isspace(buffer[i]); )   /* copy subfile name            #1 */
        subFileName[j++] = buffer[i++];
      subFileName[j] = '\0';
      level++;                          /* increment nested level counter   */
      for (i=0; i < level; )            /* graphically indicate level       */
        lvlgraph[i++] = '.';
      lvlgraph[i] = '\0';
      /* print subfile name in console message                              */
      fprintf(stderr, "Processing input from %s%s.\n", lvlgraph, subFileName);
      /* open subfile; print error and terminate if cannot open             */
      if ((subFile = fopen(subFileName, "rt")) == NULL)
      {
        fprintf(stderr, "Cannot open input file (%s).\n", subFileName);
        exit(1);
      }
      /* recursively call this function to process subordinate file         */
      processFile(subFile, level);
      fclose(subFile);          /* close subordinate file                   */
      level--;                  /* decrement nested level counter           */
      continue;                 /* continue with next record from file      */
    }
    buffer[8] = byteSave;

    /* directives not recognized, process record from file as JCL           */

    /* when compiled with gcc and executed under Cygwin & Windows/2000,
       occasionally a Windows (actually legacy of MS/PC-DOS) end of file
       character (\0x1a or \032) was copied into the output file -- the
       following code eliminates that character                             */
    for (i=0; buffer[i] != '\0'; i++)
      if (buffer[i] == '\032')
        for (j=i; buffer[j] != '\0'; j++)
          buffer[j] = buffer[j+1];

    fprintf(stdout, "%s", buffer);

  } while (1 == 1); /* end of loop for processing file contents             */

} /* processFile */

/****************************************************************************
 * main checks for file name as argument; prints syntax if omitted;
 * opens file; prints error message and terminates if cannot open;
 * calls function to process records from the file; closes file and exits
 ****************************************************************************/
int main (int argc, char *argv[])
{
  FILE *mainFile;       /* handle for opening main (argument) file          */
  int level = 0;        /* used to indicate nested file level               */

  fprintf(stderr, "JCLSTREAM v1.1 copyright Jay Moseley, CCP 2004.\n");

  /* if no argument specified, print syntax and exit                        */
  if (argc == 1)
  {
    fprintf(stderr, "Syntax: JCLSTREAM <input file name>\n");
    exit(0);
  } /* if */

  /* open file; if cannot open, print error message and terminate           */
  if ((mainFile = fopen(argv[1], "rt")) == NULL)
  {
    fprintf(stderr, "Cannot open input file (%s).\n", argv[1]);
    exit(1);
  }

  /* print main file name in console message                                */
  fprintf(stderr, "Processing input from %s.\n", argv[1]);

  /* call function to process records in the file                           */
  processFile(mainFile, level);

  fclose(mainFile);     /* close main file                                  */

  return 0;             /* exit to system                                   */

} /* main */
